#include "GPAdvancedLayerStruct.h"
#include "gputils.h"

#include <iostream>

namespace genetic_pack {

	GPAdvancedLayerStruct::GPAdvancedLayerStruct(double W, double H, double left, double right, 
		vector<GPPoint> defects): W(W), H(H), left(left), right(right), defects(defects)
	{
		// assertions
		if (compare(left, 0) < 0 || compare(left, right) > 0 || compare(right, W) > 0)
		{
			cerr << "Incorrect parameters for GPAdvancedLayerStruct" << endl;
			throw "Incorrect parameters for GPAdvancedLayerStruct";
		}

		// initialise x's and y's
		x.insert(left);
		for (unsigned int i = 0; i < defects.size(); i++)
		{
			x.insert(defects[i].x);
		}

		y.insert(0.0);
		for (unsigned int i = 0; i < defects.size(); i++)
		{
			y.insert(defects[i].y);
		}

		coveredArea = 0.0;
	}

	GPAdvancedLayerStruct::~GPAdvancedLayerStruct(void)
	{
	}

	const vector<GPRectPosition>& GPAdvancedLayerStruct::getRectangles() const
	{
		return rectangles;
	}
	
	bool GPAdvancedLayerStruct::tryToPut(const GPRectPosition& R)
	{
		// optimization check: if enough free area available
		if (coveredArea + R.GetRectangle().GetArea() > (right - left) * H)
		{
			return false;
		}
		
		// checking position and sizes
		GPPoint layerBase;
		layerBase.x = left; 
		layerBase.y = 0;
		GPRectPosition layerRect(GPRectangle(right - left, H), layerBase);
		if (isOutsideRect(R.GetBase(), layerRect) || isOutsideRect(R.GetTopRight(), layerRect))
		{
			return false;
		}

		// checking overlapping with defects and rectangles already in the layer
		for (unsigned int i = 0; i < defects.size(); i++)
		{
			if (isStrictlyInsideRect(defects[i], R))
				return false;
		}

		for (unsigned int i = 0; i < rectangles.size(); i++)
		{
			if (isOverlapped(rectangles[i], R))
				return false;
		}
		
		// adding R to rectangles
		GPRectPosition newR = R; // copy
		rectangles.push_back(newR);

		// updating x's and y's
		x.insert(newR.GetBase().x);
		x.insert(newR.GetTopRight().x);
		y.insert(newR.GetBase().y);
		y.insert(newR.GetTopRight().y);

		// updating covered area
		coveredArea += R.GetRectangle().GetArea();

		return true;
	}

} // namespace genetic_pack