#pragma once

#include <vector>
#include <set>

#include "GPPoint.h"
#include "GPRectPosition.h"

using namespace std;

namespace genetic_pack {

	// class to represent the data of the layer for AdvancedLayerAlgorithm
	class GPAdvancedLayerStruct
	{
		double W, H; // metal sheet's width and height
		double left, right; // borders of the layer
		double coveredArea; // covered area
		vector<GPPoint> defects; // defects inside the layer - just to speed up the algorithm
		vector<GPRectPosition> rectangles; // rectangles in the layer
		set<double> x;
		set<double> y;
	public:
		GPAdvancedLayerStruct(double W, double H, double left, double right, 
			vector<GPPoint> defects);
		~GPAdvancedLayerStruct(void);

		const vector<GPRectPosition>& getRectangles() const;

		double getLeft() const { return left; };
		double getRight() const { return right; };
		double getHeight() const { return H; };
		double getCoveredArea() const { return coveredArea; };

		const set<double>& getX() const { return x; };
		const set<double>& getY() const { return y; };
		bool tryToPut(const GPRectPosition& R); // tries to put R in the position specified
	};

} // namespace genetic_pack