#pragma once

#include "GPContainer.h"
#include "GPChromosome.h"
#include "GPPhenotype.h"

namespace genetic_pack {

	// class implements the genetic algorithm itself
	class GPAlgorithm
	{
		GPContainer container;
		unsigned int popSize; // size of population, i.e. number of possible solutions per generation
		unsigned int eliteSize; // elite size
		double mutationProp; // probability of mutation
		int crossoverMode;
		int layoutMethod;

		vector<GPChromosome> population; // list of permutations in current solution
		vector<GPPhenotype> phenotypes; // cache of layouts
		bool phenotypesCached; // flag: if phenotypes has already been cached

		void cachePhenotypes(); // caches phenotypes
	public:
		GPAlgorithm(const GPContainer& container, unsigned int popSize, 
			unsigned int eliteSize, double mutationProp, int crossoverMode, 
			int layoutMethod);

		void initPopulation(); // initialises population with random permutations
		void nextGeneration(); // generates next generation

		void doAllSteps(int generations); // init and then generate specified number of generations

		double GetBestFit(); // what is the best value of fitness of the current generation?
		double GetAverageFit(); // what is the average value of fitness of the current generation?

		GPChromosome GetBestChromosome(); // which chromosome encodes the best current solution

		const vector<GPChromosome>& GetPopulation() const { return population; };
	};

} // namespace genetic_pack

