#pragma once

#include <vector>

using std::vector;

namespace genetic_pack {

	// constants for crossover types
	const int JAKOBS_CROSSOVER = 0;
	const int PMX_CROSSOVER = 1;

	// represents a permutation of (0,1,2,...n-1)
	class GPChromosome
	{
	private:
		vector<unsigned int> permutation;
	public:
		GPChromosome(const vector<unsigned int> permutation);

		const vector<unsigned int>& GetPermutation() const { return permutation; }
		unsigned int operator[](unsigned int index) const;
		unsigned int size() const { return permutation.size(); };

		void doMutation();

		bool operator==(const GPChromosome& another) const;
		int find(const vector<GPChromosome>& lst);
	};

	// random permutation of length n
	GPChromosome randomChromosome(unsigned int n);

	// general crossover function, calls particular one
	GPChromosome doCrossover(const GPChromosome& a, const GPChromosome& b, int mode = JAKOBS_CROSSOVER);
	
	// particular crossover - Jakobs crossover
	GPChromosome doJakobsCrossover(const GPChromosome& a, const GPChromosome& b);

	// particular crossover - PMX
	GPChromosome doPMXCrossover(const GPChromosome& a, const GPChromosome& b);
} // namespace genetic_pack
