#include "GPContainer.h"
#include "GPRandom.h"

#include <sstream>
#include <iostream>
using namespace std;

namespace genetic_pack {
	GPContainer::GPContainer(GPRectangle metalSheet, vector<GPPoint> defects, vector<GPRectangle> rectangles): 
		metalSheet(metalSheet), defects(defects), rectangles(rectangles)
	{
		GPPoint zero = {0,0};
		GPRectPosition rectPosition(metalSheet, zero);

		// checking that all defects are inside the metal sheet
		for (vector<GPPoint>::iterator it = defects.begin(); it != defects.end(); ++it)
		{
			if (!isStrictlyInsideRect(*it, rectPosition) && !isOnRectBorder(*it, rectPosition))
			{
				std::ostringstream s;
				s << "GPContainer error: (" << it->x << "," << it->y << ") defect point is out of the sheet";
				cerr << s.str();
				throw s.str();
			}
		}

		// eliminating rectangles more that the metal sheet
		for (int i = this->rectangles.size()-1; i >= 0; i--)
		{
			if (this->rectangles[i].GetHeight() > metalSheet.GetHeight() ||
				this->rectangles[i].GetWidth() > metalSheet.GetWidth())
			{
				this->rectangles.erase(this->rectangles.begin() + i);
			}
		}
	}

	// generating a random container
	// i.e. random problem input data
	GPContainer randomContainer(double W, double H, int defectsNum, int rectNum)
	{
		GPRectangle metalSheet(W, H);
		
		vector<GPPoint> defects;
		GPPoint defect;
		for (int i = 0; i < defectsNum; i++)
		{
			defect.x = getRandom()*W;
			defect.y = getRandom()*H;
			defects.push_back(defect);
		}

		vector<GPRectangle> rectangles;
		for (int i = 0; i < rectNum; i++)
			rectangles.push_back(GPRectangle(getRandom()*W, getRandom()*H));

		return GPContainer(metalSheet, defects, rectangles);
	}

	// container output to the output stream
	// for debugging, etc.
	ostream& operator<<(ostream& out, const GPContainer& container)
	{
		out << container.GetMetalSheet().GetWidth() << " " << container.GetMetalSheet().GetHeight() << "\n";
		out << container.GetRectangles().size() << "\n";

		for (unsigned int i = 0; i < container.GetRectangles().size(); i++)
			out << container.GetRectangles()[i].GetWidth() << " " << container.GetRectangles()[i].GetHeight() << "\n";

		out << container.GetDefects().size() << "\n";

		for (unsigned int i = 0; i < container.GetDefects().size(); i++)
			out << container.GetDefects()[i].x << " " << container.GetDefects()[i].y << "\n";

		out << endl;

		return out;
	}
} // namespace genetic_pack

