#include "GPDiscreteRandom.h"

#include <sstream>

using namespace std;

namespace genetic_pack {
	GPDiscreteRandom::GPDiscreteRandom(const vector<double>& weights, int seed)
	{
		// calculating the total weight
		double totalWeight = 0.0;
		for (unsigned int i = 0; i < weights.size(); i++)
			if (weights[i] < 0)
			{
				ostringstream s;
				s << "GPDiscreteRandom error: negative weight (" << weights[i] << ")";
				throw s.str();
			}
			else
			{
				totalWeight += weights[i];
			}

		// normalising weights
		// i.e. converting them to (cumulative) probabilities
		double cumul = 0.0;
		for (unsigned int i = 0; i < weights.size(); i++)
		{
			cumul += weights[i] / totalWeight;
			cumulProbs.push_back(cumul);
		}
	}

	int GPDiscreteRandom::GetDiscrete()
	{
		double a = get();
		unsigned int i = 0;
		while (i < cumulProbs.size() && a > cumulProbs[i])
			i++;

		return i;
	}
} // namespace genetic_pack