#include "GPPhenotype.h"
#include "GPSimpleLayerAlgorithm.h"
#include "GPAdvancedLayerAlgorithm.h"
#include "gputils.h"

#include <set>
#include <iostream>

using namespace std;

namespace genetic_pack {

	GPPhenotype::GPPhenotype(const GPContainer& container, const vector<GPRectPosition>& rectangles): 
		container(container), rectangles(rectangles)
	{
	}

	// sums the areas of all covering rectangles
	double GPPhenotype::GetCoveredArea() const
	{
		double s = 0.0;

		for (unsigned int i = 0; i < rectangles.size(); i++)
		{
			s += rectangles[i].GetRectangle().GetArea();
		}

		return s;
	}

	// checks phenotype for validness
	bool GPPhenotype::IsValid() const
	{
		// checks that no two rectnagles overlap
		for (unsigned int i = 0; i < rectangles.size(); i++)
		{
			for (unsigned int j = i+1; j < rectangles.size(); j++)
			{
				if (isOverlapped(rectangles[i], rectangles[j]))
				{
					return false;
				}
			}
		}

		// cheks that no defects are covered by rectangle
		vector<GPPoint> defects(container.GetDefects());

		for (unsigned int i = 0; i < defects.size(); i++)
		{
			for (unsigned int j = 0; j < rectangles.size(); i++)
			{
				if (isStrictlyInsideRect(defects[i], rectangles[j]))
					return false;
			}
		}

		return true;
	}

	double GPPhenotype::fitnessFunction() const
	{
		// simply covered area
		return GetCoveredArea();
	}

	GPPhenotype decodeChromosome(const GPContainer& container, const GPChromosome& chromo,
		int layoutMethod)

	{
		GPLayoutMethod *method;
		switch (layoutMethod)
		{
		case ADVANCED_LAYER:
			method = new GPAdvancedLayerAlgorithm();
			break;
		default:
			method = new GPSimpleLayerAlgorithm();
			break;
		}

		GPPhenotype layout = method->decodeChromosome(container, chromo);
		delete method;

		return layout;
	}

} // namespace genetic_pack