#pragma once

#include "GPRectPosition.h"
#include "GPContainer.h"
#include "GPChromosome.h"

#include <vector>

using std::vector;

namespace genetic_pack {

	// GPPhenotype represents phenotype for GA
	// this is basically rectangles layout
	class GPPhenotype
	{
		vector<GPRectPosition> rectangles;
		GPContainer container;
	public:
		GPPhenotype(const GPContainer& container, const vector<GPRectPosition>& rectangles);

		const vector<GPRectPosition>& GetRectangles() const { return rectangles; };

		double GetCoveredArea() const;
		bool IsValid() const;

		double fitnessFunction() const;
	};

	// constants for layout methods
	const int SIMPLE_LAYER = 0;
	const int ADVANCED_LAYER = 1;

	// SimpleLayer algorithm implementation
	// decoding cromosome into phenotype
	// i.e. generating layer packing from permutation
	GPPhenotype decodeChromosome(const GPContainer& container, const GPChromosome& chromo,
		int layoutMethod);

} // namespace genetic_pack
