#include "gputils.h"

#include <vector>
#include <iostream>

using namespace std;

namespace genetic_pack {

	// compares two doubles with TOLERANCE
	// returns:
	//	-1 if a < b
	//	 0 if a = b
	//	 1 if a > b
	int compare(double a, double b)
	{
		if (a-b < -TOLERANCE)
			return -1;
		else if (a-b < TOLERANCE)
			return 0;
		else
			return 1;
	}

	// checks if the point is inside the rectangle
	bool isStrictlyInsideRect(const GPPoint& p, const GPRectPosition& rect) {
		return compare(rect.GetBase().x, p.x) < 0 &&
			compare(rect.GetTopRight().x, p.x) > 0 &&
			compare(rect.GetBase().y, p.y) < 0 &&
			compare(rect.GetTopRight().y, p.y) > 0;
	}

	bool isOnRectBorder(const GPPoint& p, const GPRectPosition& rect)
	{
		return !(isOutsideRect(p, rect) || isStrictlyInsideRect(p, rect));
	}

	// checks if the point is either iside the rectangle or on the border
	bool isOutsideRect(const GPPoint& p, const GPRectPosition& rect)
	{
		return compare(rect.GetBase().x, p.x) > 0 ||
			compare(rect.GetTopRight().x, p.x) < 0 ||
			compare(rect.GetBase().y, p.y) > 0 ||
			compare(rect.GetTopRight().y, p.y) < 0;
	}

	bool tmp(GPRectPosition r1, GPRectPosition r2);

	// checks if two rectangles overlap
	bool isOverlapped(GPRectPosition r1, GPRectPosition r2)
	{
		double x0 = r1.GetBase().x,
			y0 = r1.GetBase().y,
			x1 = r1.GetTopRight().x,
			y1 = r1.GetTopRight().y,
			x2 = r2.GetBase().x,
			y2 = r2.GetBase().y,
			x3 = r2.GetTopRight().x,
			y3 = r2.GetTopRight().y;

		bool ret = (compare(x1, x2) > 0 && compare(x3, x0) > 0) &&
			(compare(y1, y2) > 0 && compare(y3, y0) > 0);

#ifdef _DEBUG
		//if (!ret) {
		//	cout << "DEBUG: NON-overlapping rectangles\n";
		//	tmp(r1, r2);
		//}
#endif
		return ret;
	}

	bool tmp(GPRectPosition r1, GPRectPosition r2)
	{
		vector<GPPoint> vertices1 = r1.GetVertices(),
			vertices2 = r2.GetVertices();

		GPPoint centre1, centre2;
		centre1.x = 0.5 * (r1.GetBase().x + r1.GetTopRight().x);
		centre1.y = 0.5 * (r1.GetBase().y + r1.GetTopRight().y);
		vertices1.push_back(centre1);
		centre2.x = 0.5 * (r2.GetBase().x + r2.GetTopRight().x);
		centre2.y = 0.5 * (r2.GetBase().y + r2.GetTopRight().y);
		vertices2.push_back(centre2);

		bool ret = false;

		for (std::vector<GPPoint>::iterator it = vertices1.begin(); (!ret) && it != vertices1.end(); ++it)
		{
			ret = ret || isStrictlyInsideRect(*it, r2);
		}

		for (std::vector<GPPoint>::iterator it = vertices2.begin(); (!ret) && it != vertices2.end(); ++it)
		{
			ret = ret || isStrictlyInsideRect(*it, r1);
		}

		ret = ret || (r1 == r2);

#ifdef _DEBUG
		if (!ret)
			cout << "DEBUG: NON-overlapping rectangles\n";
#endif
		return ret;
	}

} //  namespace genetic_pack